﻿using System;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.TeamFoundation;
using System.Linq;

namespace Goshoom.DynamicsAX.TFS.WorkItems
{
    public class TfsProject
    {
        TfsTeamProjectCollection projectCollection;
        WorkItemStore workItemStore;
        Project project;
        VersionControlServer versionControl;

        private VersionControlServer VersionControl
        {
            get
            {
                if (versionControl == null)
                {
                    versionControl = projectCollection.GetService<VersionControlServer>();  
                }
                return versionControl;
            }
        }
        public bool HasWorkItemsCheckinPolicy
        {
            get
            {
                TeamProject teamProject = this.VersionControl.GetTeamProject(project.Name);
                PolicyEnvelope[] policies = teamProject.GetCheckinPolicies();
                return policies.Any(policy => policy.Type == "Work Items" && policy.Enabled == true);
            }
        }

        public void Init(string projectCollectionName, string projectName)
        {
            #region parameter validations
            if (string.IsNullOrEmpty(projectCollectionName))
            {
                throw new ArgumentNullException("projectCollectionName");
            }
            if (string.IsNullOrEmpty(projectName))
            {
                throw new ArgumentNullException("projectName");
            }
            #endregion

            projectCollection = new TfsTeamProjectCollection(new Uri(projectCollectionName), new UICredentialsProvider());
            projectCollection.EnsureAuthenticated();
            workItemStore = projectCollection.GetService<WorkItemStore>();
            project = workItemStore.Projects[projectName];
        }
        public QueryItemAdapter GetQuery(Guid queryId)
        {
            return new QueryDefinitionAdapter(workItemStore.GetQueryDefinition(queryId));
        }
        public QueryItemAdapter GetQueryRoot()
        {
            return QueryItemAdapter.NewFromQueryItem(project.QueryHierarchy);
        }
        public void LinkChangsetToWorkItem(int changesetId, int workItemId)
        { 
            LinkChangsetToWorkItem(changesetId, workItemId, "");
        }
        public void LinkChangsetToWorkItem(int changesetId, int workItemId, string comment)
        {
            #region parameter validations
            if (changesetId <= 0)
            {
                throw new ArgumentOutOfRangeException("changesetId");
            }
            if (workItemId <= 0)
            {
                throw new ArgumentOutOfRangeException("workItemId");
            }
            #endregion

            WorkItem workItem = workItemStore.GetWorkItem(workItemId);
            ExternalLink link = new ExternalLink(workItemStore.RegisteredLinkTypes[ArtifactLinkIds.Changeset],
                this.VersionControl.GetChangeset(changesetId).ArtifactUri.AbsoluteUri);
            link.Comment = comment;

            workItem.Open();
            workItem.Links.Add(link);
            workItem.Save();
        }
    }
}
